@@ -481,7 +481,7 @@ out:
 }
 
 /**
- * gupnp_service_proxy_send_action_list: (rename-to gupnp_service_proxy_send_action_list)
+ * gupnp_service_proxy_send_action_list:
  * @proxy: (transfer none) : A #GUPnPServiceProxy
  * @action: An action
  * @in_names: (element-type utf8) (transfer none): #GList of 'in' parameter
@@ -724,6 +724,7 @@ prepare_action_msg (GUPnPServiceProxy              *proxy,
                                   ret->msg_str->len);
 
         g_string_free (ret->msg_str, FALSE);
+        ret->msg_str = NULL;
 }
 
 static void
@@ -2065,7 +2066,7 @@ gupnp_service_proxy_get_subscribed (GUPnPServiceProxy *proxy)
 /**
  * gupnp_service_proxy_call_action_async:
  * @proxy: (transfer none): A #GUPnPServiceProxy
- * @action: An action
+ * @action: (transfer none): A #GUPnPServiceProxyAction to call
  * @cancellable: (allow-none): A #GCancellable which can be used to cancel the
  * current action call
  * @callback: (scope async): A #GAsyncReadyCallback to call when the action is
@@ -2114,7 +2115,7 @@ gupnp_service_proxy_call_action_async (GUPnPServiceProxy       *proxy,
  * Finish an asynchronous call initiated with
  * gupnp_service_proxy_call_action_async().
  *
- * Returns: %NULL, if the call had an error, the action otherwise.
+ * Returns: (nullable) (transfer none): %NULL, if the call had an error, the action otherwise.
  */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
@@ -2129,14 +2130,14 @@ gupnp_service_proxy_call_action_finish (GUPnPServiceProxy *proxy,
 /**
  * gupnp_service_proxy_call_action:
  * @proxy: (transfer none): A #GUPnPServiceProxy
- * @action: An action
+ * @action: (transfer none): An action
  * @cancellable: (allow-none): A #GCancellable which can be used to cancel the
  * current action call
  * @error: (allow-none): Return location for a #GError, or %NULL.
  *
  * Synchronously call the @action on the remote UPnP service.
  *
- * Returns: %NULL on error, @action if successful.
+ * Returns: (nullable)(transfer none): %NULL on error, @action if successful.
  */
 GUPnPServiceProxyAction *
 gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
@@ -2151,6 +2152,9 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
 
         prepare_action_msg (proxy, action, cancellable);
 
+        /* prepare_action_msg has queued the message, so remove it */
+        gupnp_service_proxy_remove_action (proxy, action);
+
         if (action->error != NULL) {
                 g_propagate_error (error, g_error_copy (action->error));
 
@@ -2167,9 +2171,6 @@ gupnp_service_proxy_call_action (GUPnPServiceProxy       *proxy,
                 soup_session_send_message (session, action->msg);
         }
 
-        /* prepare_action_msg has queued the message, so remove it */
-        gupnp_service_proxy_remove_action (proxy, action);
-
         if (action->msg->status_code == SOUP_STATUS_CANCELLED) {
                 action->error = g_error_new (G_IO_ERROR,
                                              G_IO_ERROR_CANCELLED,
